/*
 * Rationale.java
 *
 * Created on 1. mai 2007, 12:23
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package no.traceme.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 *
 * @author Gyrd Norvoll 
 */
public class Alternative extends Artifact implements Serializable {
    public static final String recognizer = "Alternative";
    public static final String traceabilityStage = "pre_rs";
    private String classType = recognizer;
   
    private String description;
    
    private ArrayList connectedArtifactCategories = new ArrayList();
    
    private Stakeholder proposingStakeholder;
    private Debate debate;
    private Set arguments = new HashSet(); // must contain a range of hashmaps, indicating whether the argument is supporting the alternative or not.
    
    
    /** Creates a new instance of Rationale */
    public Alternative() {
        super(recognizer, traceabilityStage, new Date());
        buildModel();
    }
    public Alternative(Long id, String tag, String description, Stakeholder proposingStakeholder){
        super(id, tag, traceabilityStage, recognizer, new Date());
        this.description = description;
        this.setProposingStakeholder(proposingStakeholder);
        
        buildModel();
    }
    private void buildModel(){
        HashMap t = new HashMap();
        t.put("recognizer", no.traceme.model.Argument.recognizer);
        t.put("set", arguments);
        t.put("trace", "validates");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Stakeholder.recognizer);
        t.put("object", proposingStakeholder);
        t.put("trace", "proposes");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Debate.recognizer);
        t.put("object", debate);
        t.put("trace", "discusses");
        getConnectedArtifactCategories().add(t);
    }
    
   public String toString(){
       return "Alternative";
   }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public ArrayList getConnectedArtifactCategories() {
        return connectedArtifactCategories;
    }

    public void setConnectedArtifactCategories(ArrayList connectedArtifactCategories) {
        this.connectedArtifactCategories = connectedArtifactCategories;
    }

    public String getClassType() {
        return classType;
    }

    public void setClassType(String classType) {
        this.classType = classType;
    }

    public Stakeholder getProposingStakeholder() {
        return proposingStakeholder;
    }

    public void setProposingStakeholder(Stakeholder proposingStakeholder) {
        this.proposingStakeholder = proposingStakeholder;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Stakeholder")){
                m.put("object", proposingStakeholder);
            }
            
        }
    }

    public Debate getDebate() {
        return debate;
    }

    public void setDebate(Debate debate) {
        this.debate = debate;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Debate")){
                m.put("object", debate);
            }
            
        }
    }

    public Set getArguments() {
        return arguments;
    }

    private void setArguments(Set arguments) {
        this.arguments = arguments;
    }
    public void addArgument(Argument validatingArgument){
        if(validatingArgument == null) throw new IllegalArgumentException();
        if(!(arguments.contains(validatingArgument))){
            validatingArgument.setValidatedAlternative(this);
            arguments.add(validatingArgument);
        }
    }
    public void removeArgument(Argument validatingArgument){
        if(validatingArgument == null) throw new IllegalArgumentException();
        if(arguments.contains(validatingArgument)){
            validatingArgument.setValidatedAlternative(null);
            arguments.remove(validatingArgument);
        }
    }
}
