/*
 * Rationale.java
 *
 * Created on 1. mai 2007, 12:23
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package no.traceme.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

/**
 *
 * @author Gyrd Norvoll 
 */
public class Assumption extends Artifact implements Serializable {
    public static final String recognizer = "Assumption";
    public static final String traceabilityStage = "pre_rs";
    private String classType = recognizer;
   
    private String description;
    
    private ArrayList connectedArtifactCategories = new ArrayList();
    
    private Set dependingArguments = new HashSet();
    private Set dependingDecisions = new HashSet();
    private Set dependingRationales = new HashSet();
    
    
    /** Creates a new instance of Rationale */
    public Assumption() {
        super(recognizer, traceabilityStage, new Date());
        buildModel();
    }
    public Assumption(Long id, String tag, String description){
        super(id, tag, traceabilityStage, recognizer, new Date());
        this.description = description;
        
        buildModel();
    }
    private void buildModel(){
        HashMap t = new HashMap();
        t.put("recognizer", no.traceme.model.Argument.recognizer);
        t.put("set", dependingArguments);
        t.put("trace", "depends on");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Decision.recognizer);
        t.put("set", dependingDecisions);
        t.put("trace", "depends on");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Rationale.recognizer);
        t.put("set", dependingRationales);
        t.put("trace", "depends on");
        getConnectedArtifactCategories().add(t);
    }
    
   public String toString(){
       return "Assumption";
   }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public ArrayList getConnectedArtifactCategories() {
        return connectedArtifactCategories;
    }

    public void setConnectedArtifactCategories(ArrayList connectedArtifactCategories) {
        this.connectedArtifactCategories = connectedArtifactCategories;
    }

    public String getClassType() {
        return classType;
    }

    public Set getDependingArguments() {
        return dependingArguments;
    }

    private void setDependingArguments(Set dependingArguments) {
        this.dependingArguments = dependingArguments;
    }
    public void addDependingArgument(Argument arg){
        if(arg == null) throw new IllegalArgumentException();
        if(!(dependingArguments.contains(arg))){
            arg.setDependingAssumption(this);
            dependingArguments.add(arg);
        }
    }
    public void removeDependingArgument(Argument arg){
        if(arg == null) throw new IllegalArgumentException();
        if(dependingArguments.contains(arg)){
            arg.setDependingAssumption(null);
            dependingArguments.remove(arg);
        }
    }

    public Set getDependingDecisions() {
        return dependingDecisions;
    }

    private void setDependingDecisions(Set dependingDecisions) {
        this.dependingDecisions = dependingDecisions;
    }
    public void addDependingDecision(Decision dec){
        if(dec == null) throw new IllegalArgumentException();
        if(!(dependingDecisions.contains(dec))){
            dec.setValidAssumption(this);
            dependingDecisions.add(dec);
        }
    }
    public void removeDependingDecision(Decision dec){
        if(dec == null) throw new IllegalArgumentException();
        if(dependingDecisions.contains(dec)){
            dec.setValidAssumption(null);
            dependingDecisions.remove(dec);
        }
    }
    public Set getDependingRationales() {
        return dependingRationales;
    }

    private void setDependingRationales(Set dependingRationales) {
        this.dependingRationales = dependingRationales;
    }
    public void addDependingRationale(Rationale dependingRationale){
        if(dependingRationale == null) throw new IllegalArgumentException();
        if(!(dependingRationales.contains(dependingRationale))){
            dependingRationale.setUnderlyingAssumption(this);
            dependingRationales.add(dependingRationale);
        }
    }
    public void removeDependingRationale(Rationale dependingRationale){
        if(dependingRationale == null) throw new IllegalArgumentException();
        if(dependingRationales.contains(dependingRationale)){
            dependingRationale.setUnderlyingAssumption(null);
            dependingRationales.remove(dependingRationale);
        }
    }
}
