/*
 * ChangeProposal.java
 *
 * Created on 1. mai 2007, 12:23
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package no.traceme.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

/**
 *
 * @author Gyrd Norvoll 
 */
public class ChangeProposal extends Artifact implements Serializable {
    
    public static final String recognizer = "ChangeProposal";
    public static final String traceabilityStage = "pre_rs";
    private String classType = recognizer;
   
    
    private String description;
    private boolean approved;
    private Date approvedTimestamp;
    
    private Stakeholder proposingStakeholder;
    private Rationale underlyingRationale;
    private Requirement affectedRequirement;
        
    private ArrayList connectedArtifactCategories = new ArrayList();
    
    /** Creates a new instance of ChangeProposal */
    public ChangeProposal() {
        super(recognizer, traceabilityStage, new Date());
        buildModel();
    }
    public ChangeProposal(Long id, String tag, String description, Stakeholder proposingStakeholder, Requirement affectedReq){
        super(id, tag, traceabilityStage, recognizer, new Date());
        this.setDescription(description);
        //proposingStakeholder.addProposedChange(this);
        //this.proposingStakeholder = proposingStakeholder;
        setProposingStakeholder(proposingStakeholder);
        affectedRequirement = affectedReq;
        buildModel();
        
    }
    private void buildModel(){
        HashMap t = new HashMap();
        t.put("recognizer", no.traceme.model.Requirement.recognizer);
        t.put("object", affectedRequirement);
        t.put("trace", "affects");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Rationale.recognizer);
        t.put("object", underlyingRationale);
        t.put("trace", "supports");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Stakeholder.recognizer);
        t.put("object", proposingStakeholder);
        t.put("trace", "proposes");
        getConnectedArtifactCategories().add(t);
    }
    public Requirement getAffectedRequirement() {
        return affectedRequirement;
    }

    public void setAffectedRequirement(Requirement affectedRequirement) {
        //affectedRequirement.addAffectingChangeProposal(this);
        this.affectedRequirement = affectedRequirement;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Requirement")){
                m.put("object", affectedRequirement);
            }
            
        }
    }

    public Stakeholder getProposingStakeholder() {
        return proposingStakeholder;
    }

    public void setProposingStakeholder(Stakeholder proposingStakeholder) {
        if(this.proposingStakeholder != null){
        this.proposingStakeholder.removeProposedChange(this);
        }
        proposingStakeholder.addProposedChange(this);
        this.proposingStakeholder = proposingStakeholder;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Stakeholder")){
                m.put("object", proposingStakeholder);
            }
            
        }
        
    }

     public Rationale getUnderlyingRationale() {
        return underlyingRationale;
    }

    public void setUnderlyingRationale(Rationale underlyingRationale) {
        if(underlyingRationale == null) throw new IllegalArgumentException("Null rationale");
        //Set affectedProposals = underlyingRationale.getAffectedChangeProposals();
        //if(!(affectedProposals.contains(this))){
        //    affectedProposals.add(this);
        //}
        this.underlyingRationale = underlyingRationale;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Rationale")){
                m.put("object", underlyingRationale);
            }
            
        }
    }
   

    public boolean isApproved() {
        return approved;
    }

    public void setApproved(boolean approved) {
        this.approved = approved;
    }
    
    public Date getApprovedTimestamp() {
        return approvedTimestamp;
    }

    private void setApprovedTimestamp(Date approvedTimestamp) {
        this.approvedTimestamp = approvedTimestamp;
    }
    
     public void approveChangeProposal(){
        approved = true;
        Date timeStamp = new Date();
        System.out.println("Here's the date: " + timeStamp.toString());
        setApprovedTimestamp(timeStamp);
        affectedRequirement.addAcceptedChangeProposal(this);
    }
    
   public String toString(){
       return "Change Proposal";
   } 

    public String getClassType() {
        return classType;
    }

    public void setClassType(String classType) {
        this.classType = classType;
    }

    public String getDescription() {
        //if(description == null) return "";
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public ArrayList getConnectedArtifactCategories() {
        return connectedArtifactCategories;
    }

    public void setConnectedArtifactCategories(ArrayList connectedArtifactCategories) {
        this.connectedArtifactCategories = connectedArtifactCategories;
    }
}
