/*
 * Rationale.java
 *
 * Created on 1. mai 2007, 12:23
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package no.traceme.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 *
 * @author Gyrd Norvoll 
 */
public class Debate extends Artifact implements Serializable {
    public static final String recognizer = "Debate";
    public static final String traceabilityStage = "pre_rs";
    private String classType = recognizer;
   
    private String description;
    
    private ArrayList connectedArtifactCategories = new ArrayList();
    
    private Issue startingIssue;
    private Decision endingDecision;
    private Set alternatives = new HashSet();
    
    
    
    /** Creates a new instance of Debate */
    public Debate() {
        super(recognizer, traceabilityStage, new Date());
        buildModel();
    }
    public Debate(Long id, String tag, String description){
        super(id, tag, traceabilityStage, recognizer, new Date());
        this.description = description;
        
        buildModel();
    }
    private void buildModel(){
        HashMap t = new HashMap();
        t.put("recognizer", no.traceme.model.Alternative.recognizer);
        t.put("set", alternatives);
        t.put("trace", "is discussed by");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Issue.recognizer);
        t.put("object", startingIssue);
        t.put("trace", "starts");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Decision.recognizer);
        t.put("object", endingDecision);
        t.put("trace", "ends");
        getConnectedArtifactCategories().add(t);
    }
    
   public String toString(){
       return "Debate";
   }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public ArrayList getConnectedArtifactCategories() {
        return connectedArtifactCategories;
    }

    public void setConnectedArtifactCategories(ArrayList connectedArtifactCategories) {
        this.connectedArtifactCategories = connectedArtifactCategories;
    }

    public String getClassType() {
        return classType;
    }

    public Issue getStartingIssue() {
        return startingIssue;
    }

    public void setStartingIssue(Issue startingIssue) {
        this.startingIssue = startingIssue;
        startingIssue.setStartedDebate(this);
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Issue")){
                m.put("object", startingIssue);
            }
            
        }
    }
    
    public Decision getEndingDecision() {
        return endingDecision;
    }

    public void setEndingDecision(Decision endingDecision) {
        this.endingDecision = endingDecision;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Decision")){
                m.put("object", endingDecision);
            }
            
        }
    }

    public Set getAlternatives() {
        return alternatives;
    }

    private void setAlternatives(Set alternatives) {
        this.alternatives = alternatives;
    }
    public void addAlternative(Alternative alt){
        if(alt == null) throw new IllegalArgumentException();
        if(!(alternatives.contains(alt))){
            alt.setDebate(this);
            alternatives.add(alt);
        }
    }
    public void removeAlternativet(Alternative alt){
        if(alt == null) throw new IllegalArgumentException();
        if(alternatives.contains(alt)){
            alt.setDebate(null);
            alternatives.remove(alt);
        }
    }
}
