/*
 * Rationale.java
 *
 * Created on 1. mai 2007, 12:23
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package no.traceme.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 *
 * @author Gyrd Norvoll 
 */
public class Decision extends Artifact implements Serializable {
    public static final String recognizer = "Decision";
    public static final String traceabilityStage = "pre_rs";
    private String classType = recognizer;
   
    private String description;
    private String context;
    private Date timestamp;
    
    private ArrayList connectedArtifactCategories = new ArrayList();
    
    private Set affectedRequirements = new HashSet();
    
    private Issue resolvedIssue;
    private Assumption validAssumption;
    private Stakeholder decisionMaker;
    private Alternative selectedAlternative;
    
      
    
    /** Creates a new instance of Rationale */
    public Decision() {
        super(recognizer, traceabilityStage, new Date());
        buildModel();
    }
    public Decision(Long id, String tag, String description, String context, Date timestamp){
        super(id, tag, traceabilityStage, recognizer, new Date());
        this.setDescription(description);
        this.context = context;
        this.timestamp = timestamp;
        
        buildModel();
    }
    private void buildModel(){
        HashMap t = new HashMap();
        t.put("recognizer", no.traceme.model.Requirement.recognizer);
        t.put("set", affectedRequirements);
        t.put("trace", "is affected by");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Issue.recognizer);
        t.put("object", resolvedIssue);
        t.put("trace", "is resolved by");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Assumption.recognizer);
        t.put("object", validAssumption);
        t.put("trace", "depends on");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Stakeholder.recognizer);
        t.put("object", decisionMaker);
        t.put("trace", "makes");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Alternative.recognizer);
        t.put("object", selectedAlternative);
        t.put("trace", "is selected by");
        getConnectedArtifactCategories().add(t);
    }
    
   public String toString(){
       return "Decision";
   }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public ArrayList getConnectedArtifactCategories() {
        return connectedArtifactCategories;
    }

    public void setConnectedArtifactCategories(ArrayList connectedArtifactCategories) {
        this.connectedArtifactCategories = connectedArtifactCategories;
    }

    public String getClassType() {
        return classType;
    }

    public void setClassType(String classType) {
        this.classType = classType;
    }

    public String getContext() {
        return context;
    }

    public void setContext(String context) {
        this.context = context;
    }

    public Date getTimestamp() {
        return timestamp;
    }

    public void setTimestamp(Date timestamp) {
        this.timestamp = timestamp;
    }

    public Set getAffectedRequirements() {
        return affectedRequirements;
    }

    private void setAffectedRequirements(Set affectedRequirements) {
        this.affectedRequirements = affectedRequirements;
    }
    public void addAffectedRequirement(Requirement req){
        if(req == null) throw new IllegalArgumentException();
        if(!(affectedRequirements.contains(req))){
            req.setUnderlyingDecision(this);
            affectedRequirements.add(req);
        }
    }
    public void removeDependingArgument(Requirement req){
        if(req == null) throw new IllegalArgumentException();
        if(affectedRequirements.contains(req)){
            req.setUnderlyingDecision(null);
            affectedRequirements.remove(req);
        }
    }

    public Issue getResolvedIssue() {
        return resolvedIssue;
    }

    public void setResolvedIssue(Issue resolvedIssue) {
        this.resolvedIssue = resolvedIssue;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Issue")){
                m.put("object", resolvedIssue);
            }
            
        }
    }

    public Assumption getValidAssumption() {
        return validAssumption;
    }

    public void setValidAssumption(Assumption validAssumption) {
        this.validAssumption = validAssumption;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Assumption")){
                m.put("object", validAssumption);
            }
            
        }
    }

    public Stakeholder getDecisionMaker() {
        return decisionMaker;
    }

    public void setDecisionMaker(Stakeholder decisionMaker) {
        this.decisionMaker = decisionMaker;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Stakeholder")){
                m.put("object", decisionMaker);
            }
            
        }
    }

    public Alternative getSelectedAlternative() {
        return selectedAlternative;
    }

    public void setSelectedAlternative(Alternative selectedAlternative) {
        this.selectedAlternative = selectedAlternative;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Alternative")){
                m.put("object", selectedAlternative);
            }
            
        }
    }
  
}
