/*
 * Requirement.java
 *
 * Created on 1. mai 2007, 12:35
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package no.traceme.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

/**
 *
 * @author Gyrd Norvoll 
 */
public abstract class FormalRequirement extends Artifact implements Serializable{
    public static final String recognizer = "FormalRequirement";
    public static final String traceabilityStage = "pre_frs";
    private String classType;
 
    private String title;
    private String description;
    private String gist;
    private String priority;
    private String risk;
    private String status;
    
    private Set subFunctions = new HashSet();
    private Set supraFunctions = new HashSet();
    
    private Set systemComponents = new HashSet();
    private Set designs = new HashSet();
    private Set verificationProcedures = new HashSet();
    
    private Requirement definingProseRequirement;
    private Set mandates = new HashSet();
    
    private ArrayList connectedArtifactCategories = new ArrayList();
    
    /**
     * Creates a new instance of Requirement
     */
 /*   public FormalRequirement() {
        super(recognizer, traceabilityStage, new Date());
        buildModel();
    }*/
    public FormalRequirement(String rec){
        super(rec, traceabilityStage, new Date());
        classType = rec;
        buildModel();
    }
    public FormalRequirement(Long id, String tag, String title, String description, String rec){
        super(id, tag, traceabilityStage, rec, new Date());
        classType = rec;
        //this.setTitle(title);
        //this.setDescription(description);
        buildModel();
    }
    
    private void buildModel(){
        
        HashMap m = new HashMap();
        m.put("recognizer", no.traceme.model.Requirement.recognizer);
        m.put("object", definingProseRequirement);
        m.put("trace", "defines");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.Mandate.recognizer);
        m.put("set", mandates);
        m.put("trace", "is supported by");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.FormalRequirement.recognizer);
        m.put("set", subFunctions);
        m.put("trace", "sub-functions");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.SystemComponent.recognizer);
        m.put("set", systemComponents);
        m.put("trace", "fulfills");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.Design.recognizer);
        m.put("set", designs);
        m.put("trace", "is driven by");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.VerificationProcedure.recognizer);
        m.put("set", verificationProcedures);
        m.put("trace", "is developed for");
        getConnectedArtifactCategories().add(m);
         
    }

    public String getClassType() {
        return classType;
    }
    public void setClassType(String t){
        classType = t;
    }
    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Requirement getDefiningProseRequirement() {
        return definingProseRequirement;
    }

    public void setDefiningProseRequirement(Requirement definingProseRequirement) {
        this.definingProseRequirement = definingProseRequirement;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Requirement")){
                m.put("object", definingProseRequirement);
            }
            
        }
    }

    public Set getMandates() {
        return mandates;
    }

    public void setMandate(Set mandates) {
        this.mandates = mandates;
    }
    public void addMandate(Mandate mandate){
        if(mandate == null) throw new IllegalArgumentException("Null mandate");
        mandate.addFormalRequirement(this);
        getMandates().add(mandate);
    }
    public void removeMandate(Mandate mandate){
        if(mandate == null) throw new IllegalArgumentException();
        if(mandates.contains(mandate)){
            mandate.removeFormalRequirement(this);
            mandates.remove(mandate);
        }
    }
    public ArrayList getConnectedArtifactCategories() {
        return connectedArtifactCategories;
    }

    public void setConnectedArtifactCategories(ArrayList connectedArtifactCategories) {
        this.connectedArtifactCategories = connectedArtifactCategories;
    }

    public Set getSubFunctions() {
        return subFunctions;
    }

    public void setSubFunctions(Set subFunctions) {
        this.subFunctions = subFunctions;
    }
    public void addSubFunction(FormalRequirement fr){
        if(fr == null) throw new IllegalArgumentException("Null subfunction");
        subFunctions.add(fr);
        fr.addSupraFunction(this);
    }
    public void removeSubFunction(FormalRequirement fr){
        if(fr == null) throw new IllegalArgumentException();
        if(subFunctions.contains(fr)){
            subFunctions.remove(fr);
            fr.removeSupraFunction(this);
        }
    }
    public Set getSupraFunctions() {
        return supraFunctions;
    }

    public void setSupraFunctions(Set supraFunctions) {
        this.supraFunctions = supraFunctions;
    }
    public void addSupraFunction(FormalRequirement fr){
        if(fr == null) throw new IllegalArgumentException("Null suprafunction");
        supraFunctions.add(fr);
        
    }
    public void removeSupraFunction(FormalRequirement fr){
        if(fr == null) throw new IllegalArgumentException();
        if(supraFunctions.contains(fr)){
            supraFunctions.remove(fr);
            
        }
    }

    public Set getSystemComponents() {
        return systemComponents;
    }

    public void setSystemComponents(Set systemComponents) {
        this.systemComponents = systemComponents;
    }
    public void addSystemComponent(SystemComponent systemComponent){
        if(systemComponent == null) throw new IllegalArgumentException("Null systemComponent");
        systemComponent.setFulfilledFormalRequirement(this);
        systemComponents.add(systemComponent);
        
    }
    public void removeSystemComponent(SystemComponent systemComponent){
        if(systemComponent == null) throw new IllegalArgumentException();
        if(systemComponents.contains(systemComponent)){
            systemComponent.setFulfilledFormalRequirement(null);
            systemComponents.remove(systemComponent);
            
        }
    }

    public Set getDesigns() {
        return designs;
    }

    public void setDesigns(Set designs) {
        this.designs = designs;
    }
    public void addDesign(Design design){
        if(design == null) throw new IllegalArgumentException("Null design");
        design.addDrivingFormalRequirement(this);
        designs.add(design);
        
    }
    public void removeDesign(Design design){
        if(design == null) throw new IllegalArgumentException();
        if(designs.contains(design)){
            design.removeDrivingFormalRequirement(this);
            designs.remove(design);
            
        }
    }

    public Set getVerificationProcedures() {
        return verificationProcedures;
    }

    public void setVerificationProcedures(Set verificationProcedures) {
        this.verificationProcedures = verificationProcedures;
    }
    public void addVerificationProcedure(VerificationProcedure vp){
        if(vp == null) throw new IllegalArgumentException("Null verificationProcedure");
        vp.addTargetedFormalRequirement(this);
        verificationProcedures.add(vp);
        
    }
    public void removeVerificationProcedure(VerificationProcedure vp){
        if(vp == null) throw new IllegalArgumentException("Null verificationProcedure");
        if(verificationProcedures.contains(vp)){
            vp.removeTargetedFormalRequirement(this);
            verificationProcedures.remove(vp);
            
        }
    }

    public String getGist() {
        return gist;
    }

    public void setGist(String gist) {
        this.gist = gist;
    }

    public String getPriority() {
        return priority;
    }

    public void setPriority(String priority) {
        this.priority = priority;
    }

    public String getRisk() {
        return risk;
    }

    public void setRisk(String risk) {
        this.risk = risk;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }
    public String toString(){
        return "Formal Requirement";
    }
}
