/*
 * Rationale.java
 *
 * Created on 1. mai 2007, 12:23
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package no.traceme.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

/**
 *
 * @author Gyrd Norvoll 
 */
public class Issue extends Artifact implements Serializable {
    
    
    public static final String recognizer = "Issue";
    public static final String traceabilityStage = "pre_rs";
    private String classType = recognizer;
   
    private String description;
    private boolean resolved = false;
    private Date resolvedTimestamp;
    
    private ArrayList connectedArtifactCategories = new ArrayList();
    
   private Requirement generatingRequirement;
   private Decision resolvingDecision;
   private Debate startedDebate;
    
    
    /** Creates a new instance of Rationale */
    public Issue() {
        super(recognizer, traceabilityStage, new Date());
        buildModel();
    }
    public Issue(Long id, String tag, String description){
        super(id, tag, traceabilityStage, recognizer, new Date());
        this.description = description;
        
        buildModel();
    }
    private void buildModel(){
        HashMap t = new HashMap();
        t.put("recognizer", no.traceme.model.Requirement.recognizer);
        t.put("object", generatingRequirement);
        t.put("trace", "generates");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Decision.recognizer);
        t.put("object", resolvingDecision);
        t.put("trace", "resolves");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Debate.recognizer);
        t.put("object", startedDebate);
        t.put("trace", "is started by");
        getConnectedArtifactCategories().add(t);
    }
    
   public String toString(){
       return "Issue";
   }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public ArrayList getConnectedArtifactCategories() {
        return connectedArtifactCategories;
    }

    public void setConnectedArtifactCategories(ArrayList connectedArtifactCategories) {
        this.connectedArtifactCategories = connectedArtifactCategories;
    }

    public String getClassType() {
        return classType;
    }

    public boolean isResolved() {
        return resolved;
    }

    public void setResolved(boolean resolved) {
        this.resolved = resolved;
    }

    public Requirement getGeneratingRequirement() {
        return generatingRequirement;
    }

    public void setGeneratingRequirement(Requirement generatingRequirement) {
        this.generatingRequirement = generatingRequirement;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Requirement")){
                m.put("object", generatingRequirement);
            }
            
        }
    }

    public Decision getResolvingDecision() {
        return resolvingDecision;
    }

    public void setResolvingDecision(Decision resolvingDecision) {
        this.resolvingDecision = resolvingDecision;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Decision")){
                m.put("object", resolvingDecision);
            }
            
        }
    }

    public Debate getStartedDebate() {
        return startedDebate;
    }

    public void setStartedDebate(Debate startedDebate) {
        this.startedDebate = startedDebate;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Debate")){
                m.put("object", startedDebate);
            }
            
        }
    }
    
    public void resolveIssue(Decision resolvingDecision){
        setResolvingDecision(resolvingDecision);
        resolvingDecision.addAffectedRequirement(generatingRequirement);
        resolvingDecision.setResolvedIssue(this);
        if(startedDebate!=null){
            startedDebate.setEndingDecision(resolvingDecision);
        }
        setResolvedTimestamp(new Date());
        setResolved(true);
        resolvingDecision.setTimestamp(new Date());
    }

    public Date getResolvedTimestamp() {
        return resolvedTimestamp;
    }

    public void setResolvedTimestamp(Date resolvedTimestamp) {
        this.resolvedTimestamp = resolvedTimestamp;
    }
}
