/*
 * Project.java
 *
 * Created on 4. mai 2007, 10:46
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package no.traceme.model;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;

/**
 *
 * @author Gyrd Norvoll 
 */
public class Project {
    
    private Long id;
    private String projectTitle;
    private User projectManager;
    private Date startDate;
    private Set attachedUsers = new HashSet();
    private String projectDescription;
    
    private ArrayList recentArtifacts = new ArrayList();
    
    private Hashtable artifact2traceabilityStage = new Hashtable();
    
    private PreRS preRSContainer;
    private PreFRS preFRSContainer;
    private PostFRS postFRSContainer;
    /*
    private ArrayList preRS = new ArrayList();
    private ArrayList preFRS = new ArrayList();
    private ArrayList postFRS = new ArrayList();
    */
    /*
    private Set stakeholders = new HashSet();
    private Set rationales = new HashSet();
    private Set requirements = new HashSet();
    private Set requirementDescriptors = new HashSet();
    private Set changeProposals = new HashSet();
    private Set issues = new HashSet();
    private Set arguments = new HashSet();
    private Set alternatives = new HashSet();
    private Set assumptions = new HashSet();
    private Set debates = new HashSet();
    private Set decisions = new HashSet();
    */
    /** Creates a new instance of Project */
    public Project() {
        initialise();
      
    }
    public Project(long id, String projectTitle, Date startDate){
        this.setId(id);
        this.setProjectTitle(projectTitle);
        this.setStartDate(startDate);
        initialise();
        
    }
    private void initialise(){
        
        //builds the static representation of artifacts and their belonging traceability stages
        artifact2traceabilityStage.put(Alternative.recognizer, Alternative.traceabilityStage);
        artifact2traceabilityStage.put(Argument.recognizer, Argument.traceabilityStage);
        artifact2traceabilityStage.put(Assumption.recognizer, Assumption.traceabilityStage);
        artifact2traceabilityStage.put(ChangeProposal.recognizer, ChangeProposal.traceabilityStage);
        artifact2traceabilityStage.put(Component.recognizer, Component.traceabilityStage);
        artifact2traceabilityStage.put(Debate.recognizer, Debate.traceabilityStage);
        artifact2traceabilityStage.put(Decision.recognizer, Decision.traceabilityStage);
        artifact2traceabilityStage.put(Design.recognizer, Design.traceabilityStage);
        artifact2traceabilityStage.put(DesignConstraint.recognizer, DesignConstraint.traceabilityStage);
        artifact2traceabilityStage.put(ExternalInterfaceRequirement.recognizer, ExternalInterfaceRequirement.traceabilityStage);
        artifact2traceabilityStage.put(ExternalSystem.recognizer, ExternalSystem.traceabilityStage);
        artifact2traceabilityStage.put(FormalRequirement.recognizer, FormalRequirement.traceabilityStage);
        artifact2traceabilityStage.put(FunctionalRequirement.recognizer, FunctionalRequirement.traceabilityStage);
        artifact2traceabilityStage.put(Inspection.recognizer, Inspection.traceabilityStage);
        artifact2traceabilityStage.put(Issue.recognizer, Issue.traceabilityStage);
        artifact2traceabilityStage.put(Mandate.recognizer, Mandate.traceabilityStage);
        artifact2traceabilityStage.put(Module.recognizer, Module.traceabilityStage);
        artifact2traceabilityStage.put(Package.recognizer, Package.traceabilityStage);
        artifact2traceabilityStage.put(PerformanceRequirement.recognizer, PerformanceRequirement.traceabilityStage);
        artifact2traceabilityStage.put(ProgrammingClass.recognizer, ProgrammingClass.traceabilityStage);
        artifact2traceabilityStage.put(Prototype.recognizer, Prototype.traceabilityStage);
        artifact2traceabilityStage.put(Rationale.recognizer, Rationale.traceabilityStage);
        artifact2traceabilityStage.put(Requirement.recognizer, Requirement.traceabilityStage);
        artifact2traceabilityStage.put(RequirementDescriptor.recognizer, RequirementDescriptor.traceabilityStage);
        artifact2traceabilityStage.put(Simulation.recognizer, Simulation.traceabilityStage);
        artifact2traceabilityStage.put(SoftwareSystemAttribute.recognizer, SoftwareSystemAttribute.traceabilityStage);
        artifact2traceabilityStage.put(Stakeholder.recognizer, Stakeholder.traceabilityStage);
        artifact2traceabilityStage.put(SystemComponent.recognizer, SystemComponent.traceabilityStage);
        artifact2traceabilityStage.put(VerificationProcedure.recognizer, VerificationProcedure.traceabilityStage);
        
        preRSContainer = new PreRS(this);
        preFRSContainer = new PreFRS(this);
        postFRSContainer = new PostFRS(this);
        
        //pre-RS traceability artifacts
        preRSContainer.initialise();
        //pre-FRS traceability artifacts:
        preFRSContainer.initialise();
        //post-FRS traceability artifacts:
        postFRSContainer.initialise();
        
        
    }
    
    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getProjectTitle() {
        return projectTitle;
    }

    public void setProjectTitle(String projectTitle) {
        this.projectTitle = projectTitle;
    }

    public Date getStartDate() {
        return startDate;
    }

    public void setStartDate(Date startDate) {
        this.startDate = startDate;
    }

    public Set getAttachedUsers() {
        return attachedUsers;
    }

    private void setAttachedUsers(Set attachedUsers) {
        this.attachedUsers = attachedUsers;
    }
    public void addUserToProject(User user){
        if(user == null) throw new IllegalArgumentException("Null user");
        if(!(getAttachedUsers().contains(user))){
            getAttachedUsers().add(user);
            user.getCurrentProjects().add(this);
        }
    }
    public void removeUserFromProject(User user){
        if(user == null) throw new IllegalArgumentException("Null user");
        if(getAttachedUsers().contains(user)){
            getAttachedUsers().remove(user);
            user.getCurrentProjects().remove(this);
        }
    }

    public User getProjectManager() {
        return projectManager;
    }

    public void setProjectManager(User projectManager) {
        this.projectManager = projectManager;
        projectManager.addManagingProject(this);
    }

    public String getProjectDescription() {
        return projectDescription;
    }

    public void setProjectDescription(String projectDescription) {
        this.projectDescription = projectDescription;
    }

     protected void addRecentArtifact(Artifact artifact){
        if(getRecentArtifacts().size() > 4){
            getRecentArtifacts().remove(0);
        }
        getRecentArtifacts().add(artifact);
        System.out.println("Added an artifact, current size: " + getRecentArtifacts().size());
        
    }
    protected void removeRecentArtifact(Artifact artifact){
        if(recentArtifacts.contains(artifact)){
            recentArtifacts.remove(artifact);
        }
    }

    public ArrayList getRecentArtifacts() {
        return recentArtifacts;
    }

    private void setRecentArtifacts(ArrayList recentArtifacts) {
        this.recentArtifacts = recentArtifacts;
    }
    
    /*
    
    public Set getChangeProposals() {
        return changeProposals;
    }

    private void setChangeProposals(Set changeProposals) {
        this.changeProposals = changeProposals;
    }
 
    public void addChangeProposal(ChangeProposal prop){
        if(prop == null) throw new IllegalArgumentException();
        if(!(changeProposals.contains(prop))){
            changeProposals.add(prop);
            addRecentArtifact(prop);
        }
    }
    public void removeChangeProposal(ChangeProposal prop){
        if(prop == null) throw new IllegalArgumentException();
        if(changeProposals.contains(prop)){
            changeProposals.remove(prop);
            removeRecentArtifact(prop);
        }
    }
     */
    public ArrayList getPreRS() {
        return preRSContainer.getAllArtifacts();
    }

    public void setPreRS(ArrayList preRS) {
        this.preRSContainer.setAllArtifacts(preRS);
    }
    public PreRS getPreRSContainer(){
        return preRSContainer;
    }
    public void setPreRSContainer(PreRS c){
        this.preRSContainer = c;
    }
    public ArrayList getPreFRS() {
        return preFRSContainer.getAllArtifacts();
    }

    public void setPreFRS(ArrayList preFRS) {
        this.preFRSContainer.setAllArtifacts(preFRS);
    }
    public PreFRS getPreFRSContainer() {
        return preFRSContainer;
    }

    public void setPreFRSContainer(PreFRS preFRSContainer) {
        this.preFRSContainer = preFRSContainer;
    }
    public ArrayList getPostFRS() {
        return postFRSContainer.getAllArtifacts();
    }

    public void setPostFRS(ArrayList postFRS) {
        this.postFRSContainer.setAllArtifacts(postFRS);
    }
    public PostFRS getPostFRSContainer() {
        return postFRSContainer;
    }

    public void setPostFRSContainer(PostFRS postFRSContainer) {
        this.postFRSContainer = postFRSContainer;
    }

/*
    public Set getStakeholders() {
        return stakeholders;
    }

    private void setStakeholders(Set stakeholders) {
        this.stakeholders = stakeholders;
    }

        public void addStakeholder(Stakeholder s){
        if(s == null) throw new IllegalArgumentException();
        if(!(stakeholders.contains(s))){
            stakeholders.add(s);
            addRecentArtifact(s);
        }
    }
    public void removeStakeholder(Stakeholder s){
        if(s == null) throw new IllegalArgumentException();
        if(arguments.contains(s)){
            arguments.remove(s);
            removeRecentArtifact(s);
        }
    }
    public Set getRationales() {
        return rationales;
    }

    private void setRationales(Set rationales) {
        this.rationales = rationales;
    }
    public void addRationale(Rationale r){
        if(r == null) throw new IllegalArgumentException();
        if(!(rationales.contains(r))){
            rationales.add(r);
            addRecentArtifact(r);
        }
    }
    public void removeRationale(Rationale r){
        if(r == null) throw new IllegalArgumentException();
        if(rationales.contains(r)){
            rationales.remove(r);
            removeRecentArtifact(r);
        }
    }
    public Set getRequirements() {
        return requirements;
    }

    private void setRequirements(Set requirements) {
        this.requirements = requirements;
    }
    public void addRequirement(Requirement req){
        if(req == null) throw new IllegalArgumentException();
        if(!(requirements.contains(req))){
            requirements.add(req);
            addRecentArtifact(req);
        }
    }
    public void removeRequirement(Requirement req){
        if(req == null) throw new IllegalArgumentException();
        if(requirements.contains(req)){
            requirements.remove(req);
            removeRecentArtifact(req);
        }
    }
    public Set getRequirementDescriptors() {
        return requirementDescriptors;
    }
    
    private void setRequirementDescriptors(Set requirementDescriptors) {
        this.requirementDescriptors = requirementDescriptors;
    }
    public void addRequirementDescriptor(RequirementDescriptor recDesc){
        if(recDesc == null) throw new IllegalArgumentException();
        if(!(requirementDescriptors.contains(recDesc))){
            requirementDescriptors.add(recDesc);
            addRecentArtifact(recDesc);
        }
    }
    public void removeRequirementDescriptor(RequirementDescriptor recDesc){
        if(recDesc == null) throw new IllegalArgumentException();
        if(requirementDescriptors.contains(recDesc)){
            requirementDescriptors.remove(recDesc);
            removeRecentArtifact(recDesc);
        }
    }
    
    */
    
   
/*
    public Set getIssues() {
        return issues;
    }

    public void setIssues(Set issues) {
        this.issues = issues;
    }

    public Set getArguments() {
        return arguments;
    }

    public void setArguments(Set arguments) {
        this.arguments = arguments;
    }

    public Set getAlternatives() {
        return alternatives;
    }

    public void setAlternatives(Set alternatives) {
        this.alternatives = alternatives;
    }

    public Set getAssumptions() {
        return assumptions;
    }

    public void setAssumptions(Set assumptions) {
        this.assumptions = assumptions;
    }

    public Set getDebates() {
        return debates;
    }

    public void setDebates(Set debates) {
        this.debates = debates;
    }

    public Set getDecisions() {
        return decisions;
    }

    public void setDecisions(Set decisions) {
        this.decisions = decisions;
    }
 
 */

    public Hashtable getArtifact2traceabilityStage() {
        return artifact2traceabilityStage;
    }

    public void setArtifact2traceabilityStage(Hashtable artifact2traceabilityStage) {
        this.artifact2traceabilityStage = artifact2traceabilityStage;
    }
    public String findTraceStage(String artifactType){
        if(artifact2traceabilityStage.containsKey(artifactType)){
            return (String)artifact2traceabilityStage.get(artifactType);
        }else return "";
    }
   
}
