/*
 * Rationale.java
 *
 * Created on 1. mai 2007, 12:23
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package no.traceme.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 *
 * @author Gyrd Norvoll 
 */
public class Rationale extends Artifact implements Serializable {
    public static final String recognizer = "Rationale";
    public static final String traceabilityStage = "pre_rs";
    private String classType = recognizer;
    /*private Long id;
    private String tag;*/
    private String description;
    
    private ArrayList connectedArtifactCategories = new ArrayList();
    
    private Assumption underlyingAssumption;
    
    private Set affectedRequirements = new HashSet();
    private Set affectedChangeProposals = new HashSet();
    
    /** Creates a new instance of Rationale */
    public Rationale() {
        super(recognizer, traceabilityStage, new Date());
        buildModel();
    }
    public Rationale(Long id, String tag, String description){
        super(id, tag, traceabilityStage, recognizer, new Date());
        this.description = description;
        
        buildModel();
    }
    private void buildModel(){
        HashMap t = new HashMap();
        t.put("recognizer", no.traceme.model.Requirement.recognizer);
        t.put("set", affectedRequirements);
        t.put("trace", "is affected by");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.ChangeProposal.recognizer);
        t.put("set", affectedChangeProposals);
        t.put("trace", "is affected by");
        getConnectedArtifactCategories().add(t);
        
        t = new HashMap();
        t.put("recognizer", no.traceme.model.Assumption.recognizer);
        t.put("object", underlyingAssumption);
        t.put("trace", "defines");
        
    }
    public Set getAffectedRequirements() {
        return affectedRequirements;
    }

    private void setAffectedRequirements(Set affectedRequirements) {
        this.affectedRequirements = affectedRequirements;
    }
    public void addAffectedRequirement(Requirement affectedRequirement){
        if(affectedRequirement == null) throw new IllegalArgumentException();
        if(!(affectedRequirements.contains(affectedRequirement))){
            affectedRequirement.setUnderlyingRationale(this);
            affectedRequirements.add(affectedRequirement);
        }
    }
    public void removeAffectedRequirement(Requirement affectedRequirement){
        if(affectedRequirement == null) throw new IllegalArgumentException();
        if(affectedRequirements.contains(affectedRequirement)){
            affectedRequirement.setUnderlyingRationale(null);
            affectedRequirements.remove(affectedRequirement);
        }
    }
    
    public Set getAffectedChangeProposals() {
        return affectedChangeProposals;
    }

    private void setAffectedChangeProposals(Set affectedChangeProposals) {
        this.affectedChangeProposals = affectedChangeProposals;
    }
    public void addAffectedChangeProposal(ChangeProposal affectedChangeProposal){
        if(affectedChangeProposal == null) throw new IllegalArgumentException();
        if(!(affectedRequirements.contains(affectedChangeProposal))){
            affectedChangeProposal.setUnderlyingRationale(this);
            affectedRequirements.add(affectedChangeProposal);
        }
    }
    public void removeAffectedChangeProposal(ChangeProposal affectedChangeProposal){
        if(affectedChangeProposal == null) throw new IllegalArgumentException();
        if(affectedRequirements.contains(affectedChangeProposal)){
            affectedChangeProposal.setUnderlyingRationale(null);
            affectedRequirements.remove(affectedChangeProposal);
        }
    }
    
   public String toString(){
       return "Rationale";
   }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public ArrayList getConnectedArtifactCategories() {
        return connectedArtifactCategories;
    }

    public void setConnectedArtifactCategories(ArrayList connectedArtifactCategories) {
        this.connectedArtifactCategories = connectedArtifactCategories;
    }

    public String getClassType() {
        return classType;
    }

    private void setClassType(String classType) {
        this.classType = classType;
    }

    public Assumption getUnderlyingAssumption() {
        return underlyingAssumption;
    }

    public void setUnderlyingAssumption(Assumption underlyingAssumption) {
        this.underlyingAssumption = underlyingAssumption;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Assumption")){
                m.put("object", underlyingAssumption);
            }
            
        }
    }
}
