/*
 * Requirement.java
 *
 * Created on 1. mai 2007, 12:35
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package no.traceme.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

/**
 *
 * @author Gyrd Norvoll 
 */
public class Requirement extends Artifact implements Serializable {
    public static final String recognizer = "Requirement";
    public static final String traceabilityStage = "pre_rs";
    private String classType = recognizer;
    /* private Long id;
    private String tag;*/
    private String title;
    private String description;
    
    private Stakeholder owningStakeholder;
    
    private ArrayList connectedArtifactCategories = new ArrayList();
    
    private Set formalRequirements = new HashSet();
    
    private Set affectedStakeholders = new HashSet();
    private Set requirementDescriptors = new HashSet();
    private Set acceptedChangeProposals = new HashSet();
    private Set affectingChangeProposals = new HashSet();
    private Set issues = new HashSet();

    private Rationale underlyingRationale;
    private Decision underlyingDecision;
    
    /**
     * Creates a new instance of Requirement
     */
    public Requirement() {
        super(recognizer, traceabilityStage, new Date());
        buildModel();
    }
    public Requirement(String recognizer, String traceabilityStage, Date creationDate){
        super(recognizer, traceabilityStage, creationDate);
        buildModel();
    }
    
    public Requirement(Long id, String tag, String title, String description, String traceabilityStage, String recognizer, Date creationDate){
        super(id, tag, traceabilityStage, recognizer, creationDate);
        this.setTitle(title);
        this.setDescription(description);
        buildModel();
    }
    public Requirement(Long id, String tag, String title, String description){
        super(id, tag, traceabilityStage, recognizer, new Date());
        this.setTitle(title);
        this.setDescription(description);
        buildModel();
    }
    
    private void buildModel(){
        HashMap m = new HashMap();
        m.put("recognizer", no.traceme.model.Stakeholder.recognizer);
        m.put("set", affectedStakeholders);
        m.put("trace", "is affected by");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.RequirementDescriptor.recognizer);
        m.put("set", requirementDescriptors);
        m.put("trace", "describes");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.ChangeProposal.recognizer);
        m.put("set", acceptedChangeProposals);
        m.put("trace", "is accepted by");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.Decision.recognizer);
        m.put("object", underlyingDecision);
        m.put("trace", "affects");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.Rationale.recognizer);
        m.put("object", underlyingRationale);
        m.put("trace", "defines");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.Issue.recognizer);
        m.put("set", issues);
        m.put("trace", "are generated by");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.FormalRequirement.recognizer);
        m.put("set", formalRequirements);
        m.put("trace", "are defined by");
        getConnectedArtifactCategories().add(m);
    }


    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }
    
    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Stakeholder getOwningStakeholder() {
        return owningStakeholder;
    }

    public void setOwningStakeholder(Stakeholder proposingStakeholder) {
        proposingStakeholder.addOwnedRequirement(this);
        this.owningStakeholder = proposingStakeholder;
    }

    public Set getRequirementDescriptors() {
        return requirementDescriptors;
    }

    private void setRequirementDescriptors(Set requirementDescriptors) {
        this.requirementDescriptors = requirementDescriptors;
    }
    
    public void addRequirementDescriptor(RequirementDescriptor reqDesc){
        if(reqDesc == null) throw new IllegalArgumentException();
        if(!(requirementDescriptors.contains(reqDesc))){
            reqDesc.setDescribedRequirement(this);
            requirementDescriptors.add(reqDesc);
        }
    }
    public void removeRequirementDescriptor(RequirementDescriptor reqDesc){
        if(reqDesc == null) throw new IllegalArgumentException();
        if(requirementDescriptors.contains(reqDesc)){
            reqDesc.setDescribedRequirement(null);
            requirementDescriptors.remove(reqDesc);
        }
    }
    
    public Set getAcceptedChangeProposals() {
        return acceptedChangeProposals;
    }

    private void setAcceptedChangeProposals(Set acceptedChangeProposals) {
        this.acceptedChangeProposals = acceptedChangeProposals;
    }
    public void addAcceptedChangeProposal(ChangeProposal proposal){
        if(proposal == null) throw new IllegalArgumentException("Null proposal");
        getAcceptedChangeProposals().add(proposal);
        if(owningStakeholder!= null)owningStakeholder.removeOwnedRequirement(this);
        setOwningStakeholder(proposal.getProposingStakeholder());
    }
    public void removeAcceptedChangeProposal(ChangeProposal proposal){
        if(proposal == null) throw new IllegalArgumentException();
        if(getAcceptedChangeProposals().contains(proposal)){
            //proposal.setAffectedRequirement(null);
            getAcceptedChangeProposals().remove(proposal);
        }
    }
    public Set getAffectedStakeholders() {
        return affectedStakeholders;
    }

    private void setAffectedStakeholders(Set affectedStakeholders) {
        this.affectedStakeholders = affectedStakeholders;
    }
    public void addAffectedStakeholder(Stakeholder stakeholder){
        if(stakeholder == null) throw new IllegalArgumentException("Null stakeholder");
        Set affectingRequirements = stakeholder.getAffectingRequirements();
        if(!(affectingRequirements.contains(this))){
            affectingRequirements.add(this);
        }
         if(!(getAffectedStakeholders().contains(stakeholder))){
            getAffectedStakeholders().add(stakeholder);
        }
        
    }

    public Rationale getUnderlyingRationale() {
        return underlyingRationale;
    }

    public void setUnderlyingRationale(Rationale underlyingRationale) {
        if(underlyingRationale == null) throw new IllegalArgumentException("Null rationale");
        Set affectedRequirements = underlyingRationale.getAffectedRequirements();
        if(!(affectedRequirements.contains(this))){
            affectedRequirements.add(this);
        }
        this.underlyingRationale = underlyingRationale;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Rationale")){
                m.put("object", underlyingRationale);
            }
            
        }
    }
    
   public String toString(){
       return recognizer;
   }

   

    public String getClassType() {
        return classType;
    }

    public ArrayList getConnectedArtifactCategories() {
        return connectedArtifactCategories;
    }

    public void setConnectedArtifactCategories(ArrayList connectedArtifactCategories) {
        this.connectedArtifactCategories = connectedArtifactCategories;
    }

    public Decision getUnderlyingDecision() {
        return underlyingDecision;
    }

    public void setUnderlyingDecision(Decision underlyingDecision) {
        this.underlyingDecision = underlyingDecision;
        Iterator i = connectedArtifactCategories.iterator();
        while(i.hasNext()){
            HashMap m = (HashMap) i.next();
            if(m.get("recognizer").toString().equalsIgnoreCase("Decision")){
                m.put("object", underlyingDecision);
            }
            
        }
    }

    public Set getAffectingChangeProposals() {
        return affectingChangeProposals;
    }

    private void setAffectingChangeProposals(Set affectingChangeProposals) {
        this.affectingChangeProposals = affectingChangeProposals;
    }
    public void addAffectingChangeProposal(ChangeProposal proposal){
        if(proposal == null) throw new IllegalArgumentException("Null proposal");
        proposal.setAffectedRequirement(this);
        getAffectingChangeProposals().add(proposal);
    }
    public void removeAffectingChangeProposal(ChangeProposal proposal){
        if(proposal == null) throw new IllegalArgumentException();
        if(getAffectingChangeProposals().contains(proposal)){
            proposal.setAffectedRequirement(null);
            getAffectingChangeProposals().remove(proposal);
        }
    }

    public Set getIssues() {
        return issues;
    }

    private void setIssues(Set issues) {
        this.issues = issues;
    }
    public void addIssue(Issue issue){
        if(issue == null) throw new IllegalArgumentException("Null issue");
        issue.setGeneratingRequirement(this);
        getIssues().add(issue);
    }
    public void removeIssue(Issue issue){
        if(issue == null) throw new IllegalArgumentException();
        if(issues.contains(issue)){
            issue.setGeneratingRequirement(null);
            issues.remove(issue);
        }
    }

    public Set getFormalRequirements() {
        return formalRequirements;
    }

    public void setFormalRequirements(Set formalRequirements) {
        this.formalRequirements = formalRequirements;
    }
    public void addFormalRequirement(FormalRequirement formalRequirement){
        if(formalRequirement == null) throw new IllegalArgumentException("Null formalRequirement");
        formalRequirement.setDefiningProseRequirement(this);
        formalRequirements.add(formalRequirement);
    }
    public void removeFormalRequirement(FormalRequirement formalRequirement){
        if(formalRequirement == null) throw new IllegalArgumentException();
        if(formalRequirements.contains(formalRequirement)){
            formalRequirement.setDefiningProseRequirement(null);
            formalRequirements.remove(formalRequirement);
        }
    }
}
