/*
 * Stakeholder.java
 *
 * Created on 1. mai 2007, 12:23
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package no.traceme.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

/**
 *
 * @author Gyrd Norvoll 
 */
public class Stakeholder extends Artifact implements Serializable {
    public static final String recognizer = "Stakeholder";
    public static final String traceabilityStage = "pre_rs";
    /*private Long id;
    private String tag;*/
    private String classType = recognizer;
    private String name;
    private String position;
    private String authority;
    private String category;
    private String phone;
    private String email;
    private String office;
    private String company;
    
    private ArrayList connectedArtifactCategories = new ArrayList();    
    
    private Set affectingRequirements = new HashSet();
    private Set ownedRequirements = new HashSet();
    private Set authoredRequirementDescriptors = new HashSet();
    private Set proposedChanges = new HashSet();
    private Set proposedAlternatives = new HashSet();
    private Set reachedDecisions = new HashSet();
    private Set proposedArguments = new HashSet();
    
    /** Creates a new instance of Stakeholder */
    public Stakeholder() {
        super(recognizer, traceabilityStage, new Date());
        buildModel();
    }
    public Stakeholder(String[] parameters){
        super(new Long(parameters[0]), parameters[1], traceabilityStage, recognizer, new Date());
        this.name = parameters[2];
        this.position = parameters[3];
        this.authority = parameters[4];
        this.category = parameters[5];
        this.phone = parameters[6];
        this.email = parameters[7];
        this.office = parameters[8];
        this.company = parameters[9];
        
        buildModel();
    }
    
    public Stakeholder(Long id, String tag, String name, String position, String authority, String category, String phone, String email, String office, String company){
        super(id, tag, traceabilityStage, recognizer, new Date());
        this.name = name;
        this.position = position;
        this.authority = authority;
        this.category = category;
        this.phone = phone;
        this.email = email;
        this.office = office;
        this.company = company;
        //building the model
        buildModel();
        
        
       
    }
    private void buildModel(){
        HashMap m = new HashMap();
        m.put("recognizer", no.traceme.model.Requirement.recognizer);
        m.put("set", affectingRequirements);
        m.put("trace", "affects");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.RequirementDescriptor.recognizer);
        m.put("set", authoredRequirementDescriptors);
        m.put("trace", "is authored by");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.ChangeProposal.recognizer);
        m.put("set", proposedChanges);
        m.put("trace", "is proposed by");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.Alternative.recognizer);
        m.put("set", proposedAlternatives);
        m.put("trace", "is proposed by");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.Decision.recognizer);
        m.put("set", reachedDecisions);
        m.put("trace", "is made by");
        getConnectedArtifactCategories().add(m);
        m = new HashMap();
        m.put("recognizer", no.traceme.model.Argument.recognizer);
        m.put("set", proposedArguments);
        m.put("trace", "is proposed by");
        getConnectedArtifactCategories().add(m);
    }



    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getPosition() {
        return position;
    }

    public void setPosition(String position) {
        this.position = position;
    }

    public String getAuthority() {
        return authority;
    }

    public void setAuthority(String authority) {
        this.authority = authority;
    }

    public String getCategory() {
        return category;
    }

    public void setCategory(String category) {
        this.category = category;
    }

    public String getPhone() {
        return phone;
    }

    public void setPhone(String phone) {
        this.phone = phone;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getOffice() {
        return office;
    }

    public void setOffice(String office) {
        this.office = office;
    }

    public String getCompany() {
        return company;
    }

    public void setCompany(String company) {
        this.company = company;
    }

    public Set getAffectingRequirements() {
        return affectingRequirements;
    }

    private void setAffectingRequirements(Set affectingRequirements) {
        this.affectingRequirements = affectingRequirements;
    }
  
    public Set getAuthoredRequirementDescriptors() {
        return authoredRequirementDescriptors;
    }

    private void setAuthoredRequirementDescriptors(Set authoredRequirementDescriptors) {
        this.authoredRequirementDescriptors = authoredRequirementDescriptors;
    }
    public void addAuthoredRequirementDescriptor(RequirementDescriptor descriptor){
        if(descriptor==null) throw new IllegalArgumentException("Null AuthoredDescriptor");
        if(descriptor.getAuthoringStakeholder() != null) 
            descriptor.getAuthoringStakeholder().getAuthoredRequirementDescriptors().remove(descriptor);
        descriptor.setAuthoringStakeholder(this);
        authoredRequirementDescriptors.add(descriptor);
    }

    public Set getOwnedRequirements() {
        return ownedRequirements;
    }

    private void setOwnedRequirements(Set ownedRequirements) {
        this.ownedRequirements = ownedRequirements;
    }
    public void addOwnedRequirement(Requirement owned){
        if(owned == null) throw new IllegalArgumentException("Null OwnedRequirement");
        //if(owned.getOwningStakeholder() != null)
        //   owned.getOwningStakeholder().getOwnedRequirements().remove(owned);
        //owned.setOwningStakeholder(this);
        getOwnedRequirements().add(owned);
    }
    public void removeOwnedRequirement(Requirement owned){
        if(ownedRequirements.contains(owned)){
            ownedRequirements.remove(owned);
        }
    }
   public String toString(){
       return recognizer;
   }

    public String getClassType() {
        return classType;
    }

    public ArrayList getConnectedArtifactCategories() {
        return connectedArtifactCategories;
    }

    public void setConnectedArtifactCategories(ArrayList connectedArtifactCategories) {
        this.connectedArtifactCategories = connectedArtifactCategories;
    }

    public Set getProposedChanges() {
        return proposedChanges;
    }

    public void setProposedChanges(Set proposedChanges) {
        this.proposedChanges = proposedChanges;
    }
    public void removeProposedChange(ChangeProposal cp){
        if(proposedChanges.contains(cp)){
            proposedChanges.remove(cp);
        }
    }
    public void addProposedChange(ChangeProposal cp){
        if(!(proposedChanges.contains(cp))){
            proposedChanges.add(cp);
        }
    }

    public Set getProposedAlternatives() {
        return proposedAlternatives;
    }

    public void setProposedAlternatives(Set proposedAlternatives) {
        this.proposedAlternatives = proposedAlternatives;
    }
    public void addProposedAlternative(Alternative proposedAlternative){
        if(proposedAlternative == null) throw new IllegalArgumentException();
        if(!(proposedAlternatives.contains(proposedAlternative))){
            proposedAlternative.setProposingStakeholder(this);
            proposedAlternatives.add(proposedAlternative);
        }
    }
    public void removeProposedAlternative(Alternative proposedAlternative){
        if(proposedAlternative == null) throw new IllegalArgumentException();
        if(proposedAlternatives.contains(proposedAlternative)){
            proposedAlternative.setProposingStakeholder(null);
            proposedAlternatives.remove(proposedAlternative);
        }
    }

    public Set getReachedDecisions() {
        return reachedDecisions;
    }

    public void setReachedDecisions(Set reachedDecisions) {
        this.reachedDecisions = reachedDecisions;
    }
    public void addReachedDecision(Decision reachedDecision){
        if(reachedDecision == null) throw new IllegalArgumentException();
        if(!(reachedDecisions.contains(reachedDecision))){
            reachedDecision.setDecisionMaker(this);
            reachedDecisions.add(reachedDecision);
        }
    }
    public void removeReachedDecision(Decision reachedDecision){
        if(reachedDecision == null) throw new IllegalArgumentException();
        if(reachedDecisions.contains(reachedDecision)){
            reachedDecision.setDecisionMaker(null);
            reachedDecisions.remove(reachedDecision);
        }
    }

    public Set getProposedArguments() {
        return proposedArguments;
    }

    private void setProposedArguments(Set proposedArguments) {
        this.proposedArguments = proposedArguments;
    }
    public void addProposedArgument(Argument proposedArgument){
        if(proposedArgument == null) throw new IllegalArgumentException();
        if(!(proposedArguments.contains(proposedArgument))){
            proposedArgument.setProposingStakeholder(this);
            proposedArguments.add(proposedArgument);
        }
    }
    public void removeProposedArgument(Argument proposedArgument){
        if(proposedArgument == null) throw new IllegalArgumentException();
        if(proposedArguments.contains(proposedArgument)){
            proposedArgument.setProposingStakeholder(null);
            proposedArguments.remove(proposedArgument);
        }
    }
}
